@@ -1,5 +1,6 @@
 from datetime import datetime
 
+from django.contrib.contenttypes.models import ContentType
 from django.db import models
 from django.urls import reverse
 from django.utils.text import slugify
@@ -7,14 +8,26 @@ from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
 
 from ckeditor.fields import RichTextField
-from django_iban.fields import IBANField
+from colorfield.fields import ColorField
+from payments import PurchasedItem
 
+from aleksis.apps.tezor.models.base import Client
+from aleksis.apps.tezor.models.invoice import Invoice, InvoiceGroup
 from aleksis.core.mixins import ExtensibleModel
 from aleksis.core.models import Group, Person
 from aleksis.core.util.core_helpers import generate_random_code, get_site_preferences
 from aleksis.core.util.email import send_email
 
 
+class RegistrationState(ExtensibleModel):
+
+    name = models.CharField(verbose_name=_("Name"), max_length=255)
+    colour = ColorField(blank=True, verbose_name=_("Colour"))
+
+    def __str__(self) -> str:
+        return self.name
+
+
 class Terms(ExtensibleModel):
     title = models.CharField(max_length=255, verbose_name=_("Title"))
     term = RichTextField(verbose_name=_("Term"))
@@ -108,7 +121,11 @@ class Event(ExtensibleModel):
     information = RichTextField(verbose_name=_("Information about the event"))
     terms = models.ManyToManyField(Terms, verbose_name=_("Terms"), related_name="event", blank=True)
     info_mailings = models.ManyToManyField(
-        InfoMailing, verbose_name=_("Info mailings"), related_name="events", through="EventInfoMailingThrough", blank=True
+        InfoMailing,
+        verbose_name=_("Info mailings"),
+        related_name="events",
+        through="EventInfoMailingThrough",
+        blank=True,
     )
 
     def save(self, *args, **kwargs):
@@ -184,7 +201,7 @@ class Voucher(ExtensibleModel):
         verbose_name = _("Vouchers")
         verbose_name_plural = _("Vouchers")
 
-    code = models.CharField(max_length=8, blank=True, default="")
+    code = models.CharField(max_length=255, blank=True, default="")
     event = models.ForeignKey(
         Event,
         related_name="vouchers",
@@ -249,14 +266,85 @@ class EventRegistration(ExtensibleModel):
         related_name="registrations",
     )
 
-    accept_sepa = models.BooleanField(verbose_name=_("SEPA direct debit"))
-    iban = IBANField(
-        verbose_name=_("IBAN (for SEPA direct debit)"),
-        enforce_database_constraint=True,
-        null=True,
-        blank=True,
+    states = models.ManyToManyField(
+        RegistrationState, verbose_name=_("States"), related_name="registrations"
     )
 
+    def get_invoice(self):
+        # FIXME Maybe do not hard-code this
+        client, __ = Client.objects.get_or_create(name="Teckids e.V.")
+        group, __ = InvoiceGroup.objects.get_or_create(
+            name="Hack'n'Fun-Veranstaltungen",
+            client=client,
+            defaults={
+                "template_name": "paweljong/invoice_pdf.html",
+            },
+        )
+
+        invoice, __ = Invoice.objects.get_or_create(
+            for_content_type=ContentType.objects.get_for_model(self),
+            for_object_id=self.pk,
+            defaults={
+                "group": group,
+                "number": f"HNF-{self.date_registered.strftime('%Y-%m')}-{self.id}",
+                "currency": "EUR",
+                "total": self._get_total_amount()[0],
+                "tax": self._get_total_amount()[1],
+                "description": _("Participation of {} in event {}").format(
+                    self.person.addressing_name, self.event.display_name
+                ),
+                "billing_first_name": self.person.first_name,
+                "billing_last_name": self.person.last_name,
+                "billing_address_1": f"{self.person.street} {self.person.housenumber}",
+                "billing_city": self.person.place,
+                "billing_postcode": self.person.postal_code,
+                "billing_email": self.person.email,
+            },
+        )
+
+        return invoice
+
+    def get_purchased_items(self):
+        # FIXME Maybe do not hard-code the tax rate and currency
+        # First, return main amount
+        yield PurchasedItem(
+            name=self.event.display_name,
+            quantity=1,
+            price=self.event.cost,
+            currency="EUR",
+            sku="",
+            tax_rate=7,
+        )
+
+        # If a dnoation was made, add it
+        if self.donation:
+            yield PurchasedItem(
+                name=_("Social Sponsoring / Extra Donation"),
+                quantity=1,
+                price=self.donation,
+                currency="EUR",
+                sku="",
+                tax_rate=0,
+            )
+
+        # If a voucher was used, add it
+        if self.voucher:
+            yield PurchasedItem(
+                name=_("Voucher / Granted discount"),
+                quantity=1,
+                price=-1 * self.voucher.discount * self.event.cost / 100,
+                currency="EUR",
+                sku="",
+                tax_rate=7,
+            )
+
+    def _get_total_amount(self):
+        total, tax = 0, 0
+        for item in self.get_purchased_items():
+            total += item.price
+            tax += item.price / (item.tax_rate + 100) * item.tax_rate
+        return total, tax
+
     def __str__(self) -> str:
         return f"{self.event}, {self.person.first_name} {self.person.last_name}"
 
