@@ -11,7 +11,7 @@ from django.utils.decorators import method_decorator
 from django.utils.text import slugify
 from django.utils.translation import ugettext as _
 from django.views.decorators.cache import never_cache
-from django.views.generic import TemplateView
+from django.views.generic import FormView, TemplateView
 from django.views.generic.detail import DetailView
 
 import reversion
@@ -27,6 +27,7 @@ from aleksis.core.mixins import AdvancedCreateView, AdvancedDeleteView, Advanced
 from aleksis.core.models import Activity, Person
 from aleksis.core.util import messages
 from aleksis.core.util.core_helpers import get_site_preferences, objectgetter_optional
+from aleksis.core.util.predicates import queryset_rules_filter
 
 from .filters import EventFilter, EventRegistrationFilter, VoucherFilter
 from .forms import (
@@ -36,12 +37,15 @@ from .forms import (
     EditTermForm,
     EditVoucherForm,
     GenerateListForm,
+    RegistrationNotificationForm,
+    RegistrationStatesForm,
 )
-from .models import Event, EventRegistration, InfoMailing, Terms, Voucher
+from .models import Event, EventRegistration, InfoMailing, RegistrationState, Terms, Voucher
 from .tables import (
     EventRegistrationsTable,
     InfoMailingsTable,
     ManageEventsTable,
+    RegistrationStatesTable,
     TermsTable,
     VouchersTable,
 )
@@ -53,7 +57,7 @@ User = get_user_model()
 class CreateEventView(PermissionRequiredMixin, AdvancedCreateView):
     form_class = EditEventForm
     model = Event
-    permission_required = "paweljong.change_event"
+    permission_required = "paweljong.create_events_rule"
     template_name = "paweljong/event/create.html"
     success_url = reverse_lazy("manage_events")
     success_message = _("The event has been saved.")
@@ -64,14 +68,14 @@ class EditEventView(PermissionRequiredMixin, RevisionMixin, AdvancedEditView):
     form_class = EditEventForm
     model = Event
     slug_field = "slug"
-    permission_required = "paweljong.change_event"
+    permission_required = "paweljong.change_event_rule"
     context_object_name = "manage_events"
     template_name = "paweljong/event/edit.html"
     success_url = reverse_lazy("manage_events")
     success_message = _("The event has been saved.")
 
 
-@permission_required("paweljong.change_events")
+@permission_required("paweljong.view_events_rule")
 def manage_events(request: HttpRequest) -> HttpResponse:
     """List view listing all registrations."""
     context = {}
@@ -92,7 +96,7 @@ def manage_events(request: HttpRequest) -> HttpResponse:
     return render(request, "paweljong/event/manage.html", context)
 
 
-@permission_required("paweljong.view_vouchers")
+@permission_required("paweljong.view_vouchers_rule")
 def vouchers(request):
     context = {}
 
@@ -111,7 +115,7 @@ def vouchers(request):
     return render(request, "paweljong/voucher/list.html", context)
 
 
-@permission_required("paweljong.generate_lists")
+@permission_required("paweljong.generate_lists_rule")
 def generate_lists(request: HttpRequest) -> HttpResponse:
     context = {}
 
@@ -131,13 +135,15 @@ def generate_lists(request: HttpRequest) -> HttpResponse:
     return render(request, "paweljong/print/manage.html", context)
 
 
-@permission_required("paweljong.view_registrations")
+@permission_required("paweljong.view_registrations_rule")
 def registrations(request: HttpRequest) -> HttpResponse:
     """List view listing all registrations."""
     context = {}
 
     # Get all registrations
-    registrations = EventRegistration.objects.all()
+    registrations = queryset_rules_filter(
+        request.user, EventRegistration.objects.all(), "paweljong.manage_registrations_rule"
+    )
 
     # Get filter
     registrations_filter = EventRegistrationFilter(request.GET, queryset=registrations)
@@ -157,9 +163,9 @@ class EventRegistrationCreateView(PermissionRequiredMixin, AdvancedCreateView):
 
     model = EventRegistration
     form_class = EditEventRegistrationForm
-    permission_required = "paweljong.manage_registration"
+    permission_required = "paweljong.create_eventregistration_rule"
     template_name = "paweljong/event_registration/create.html"
-    success_url = reverse_lazy("event_registrations")
+    success_url = reverse_lazy("registrations")
     success_message = _("The event registration has been created.")
 
 
@@ -169,14 +175,14 @@ class EventRegistrationEditView(PermissionRequiredMixin, AdvancedEditView):
 
     model = EventRegistration
     form_class = EditEventRegistrationForm
-    permission_required = "paweljong.manage_eventregistration"
+    permission_required = "paweljong.change_eventregistration_rule"
     template_name = "paweljong/event_registration/edit.html"
-    success_url = reverse_lazy("event_registrations")
+    success_url = reverse_lazy("registrations")
     success_message = _("The event registration has been saved.")
 
 
 @permission_required(
-    "paweljong.manage_registrations",
+    "paweljong.change_eventregistration_rule",
     fn=objectgetter_optional(EventRegistration, None, False),
 )
 def edit_registration(request: HttpRequest, pk) -> HttpResponse:
@@ -202,7 +208,7 @@ def edit_registration(request: HttpRequest, pk) -> HttpResponse:
     return render(request, "paweljong/event_registration/edit.html", context)
 
 
-@permission_required("paweljong.is_own_voucher", fn=objectgetter_optional(Voucher, None, False))
+@permission_required("paweljong.view_voucher_rule", fn=objectgetter_optional(Voucher, None, False))
 def print_voucher(request: HttpRequest, pk) -> HttpResponse:
     context = {}
 
@@ -216,7 +222,7 @@ class EventRegistrationDetailView(PermissionRequiredMixin, DetailView):
     """Detail view for an application instance."""
 
     context_object_name = "registration"
-    permission_required = "paweljong.view_registration"
+    permission_required = "paweljong.view_registration_rule"
     template_name = "paweljong/event_registration/full.html"
 
     def get_queryset(self):
@@ -227,7 +233,7 @@ class EventRegistrationDeleteView(PermissionRequiredMixin, AdvancedDeleteView):
     """Delete view for registrations."""
 
     model = EventRegistration
-    permission_required = "paweljong.delete_eventregistration"
+    permission_required = "paweljong.delete_eventregistration_rule"
     template_name = "core/pages/delete.html"
     success_url = reverse_lazy("registrations")
     success_message = _("The registration has been deleted.")
@@ -239,7 +245,7 @@ class VoucherCreateView(PermissionRequiredMixin, AdvancedCreateView):
 
     model = Voucher
     form_class = EditVoucherForm
-    permission_required = "paweljong.add_voucher"
+    permission_required = "paweljong.create_voucher_rule"
     template_name = "paweljong/voucher/create.html"
     success_url = reverse_lazy("vouchers")
     success_message = _("The voucher has been created.")
@@ -251,7 +257,7 @@ class VoucherEditView(PermissionRequiredMixin, AdvancedEditView):
 
     model = Voucher
     form_class = EditVoucherForm
-    permission_required = "paweljong.edit_voucher"
+    permission_required = "paweljong.change_voucher_rule"
     template_name = "paweljong/voucher/edit.html"
     success_url = reverse_lazy("vouchers")
     success_message = _("The voucher has been saved.")
@@ -261,7 +267,7 @@ class VoucherDeleteView(PermissionRequiredMixin, AdvancedDeleteView):
     """Delete view for vouchers."""
 
     model = Voucher
-    permission_required = "paweljong.delete_voucher"
+    permission_required = "paweljong.delete_voucher_rule"
     template_name = "core/pages/delete.html"
     success_url = reverse_lazy("vouchers")
     success_message = _("The voucher has been deleted.")
@@ -423,9 +429,9 @@ class RegisterEventWizardView(SessionWizardView):
         elif self.steps.current == "financial":
             context["info_title"] = _("Payment")
             context["info_text"] = _(
-                "By default, we will send you an invoice, which you or your parents "
-                "can pay by bank transfer. You can also choose to pay by direct debit – "
-                "please make sure to enter exactly what your parents tell you."
+                "Please decide with your parents how you want to pay. In this step, "
+                "you only select a payment method. The real payment will be done "
+                "in a separate step, after the registration is complete."
             )
         elif self.steps.current == "consent":
             context["info_title"] = _("Consent")
@@ -611,8 +617,6 @@ class RegisterEventWizardView(SessionWizardView):
             person=person,
             medical_information=cleaned_data_additional["medical_information"],
             donation=cleaned_data_financial["donation"],
-            accept_sepa=cleaned_data_financial["accept_sepa"],
-            iban=cleaned_data_financial["iban"],
             school=cleaned_data_contact_details["school"],
             school_class=cleaned_data_contact_details["school_class"],
             school_place=cleaned_data_contact_details["school_place"],
@@ -645,18 +649,23 @@ class RegisterEventWizardView(SessionWizardView):
             else:
                 messages.error(self.request, _("You entered an invalid voucher code!"))
 
+        invoice = registration.get_invoice()
+        invoice.variant = cleaned_data_financial["payment_method"]
+        invoice.save()
+
         context = {}
         context["registration"] = registration
 
         send_templated_mail(
             template_name="event_registered",
             from_email=get_site_preferences()["mail__address"],
-            recipient_list=["orga@teckids.org"],
+            recipient_list=[get_site_preferences()["paweljong__event_notification_recipient"]],
             headers={
                 "reply_to": [
                     person.email,
                     person.guardians.first().email,
                 ],
+                "X-Zammad-Customer-Email": person.email,
             },
             context=context,
         )
@@ -677,15 +686,16 @@ class RegisterEventWizardView(SessionWizardView):
             user=person,
         )
 
-        return redirect("index")
+        return redirect("do_payment", invoice.token)
 
 
-class EventFullView(DetailView):
+class EventFullView(PermissionRequiredMixin, DetailView):
 
     model = Event
     slug_field = "slug"
     template_name = "paweljong/event/full.html"
     object_context_name = "event"
+    permission_required = "paweljong.view_event_rule"
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
@@ -695,12 +705,13 @@ class EventFullView(DetailView):
         return context
 
 
-class RegisterEventStart(DetailView):
+class RegisterEventStart(PermissionRequiredMixin, DetailView):
 
     model = Event
     slug_field = "slug"
     template_name = "paweljong/event/register_start.html"
     object_context_name = "event"
+    permission_required = "paweljong.view_event_rule"
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
@@ -714,7 +725,7 @@ class TermListView(PermissionRequiredMixin, SingleTableView):
 
     model = Terms
     table_class = TermsTable
-    permission_required = "paweljong.view_terms"
+    permission_required = "paweljong.view_terms_rule"
     template_name = "paweljong/term/list.html"
 
 
@@ -724,7 +735,7 @@ class TermCreateView(PermissionRequiredMixin, AdvancedCreateView):
 
     model = Terms
     form_class = EditTermForm
-    permission_required = "paweljong.add_terms"
+    permission_required = "paweljong.create_terms_rule"
     template_name = "paweljong/term/create.html"
     success_url = reverse_lazy("terms")
     success_message = _("The term has been created.")
@@ -736,7 +747,7 @@ class TermEditView(PermissionRequiredMixin, AdvancedEditView):
 
     model = Terms
     form_class = EditTermForm
-    permission_required = "paweljong.edit_terms"
+    permission_required = "paweljong.change_terms_rule"
     template_name = "paweljong/term/edit.html"
     success_url = reverse_lazy("terms")
     success_message = _("The term has been saved.")
@@ -783,7 +794,7 @@ class InfoMailingListView(PermissionRequiredMixin, SingleTableView):
 
     model = InfoMailing
     table_class = InfoMailingsTable
-    permission_required = "paweljong.view_info_mailing"
+    permission_required = "paweljong.view_info_mailings_rule"
     template_name = "paweljong/info_mailing/list.html"
 
 
@@ -793,7 +804,7 @@ class InfoMailingCreateView(PermissionRequiredMixin, AdvancedCreateView):
 
     model = InfoMailing
     form_class = EditInfoMailingForm
-    permission_required = "paweljong.add_info_mailing"
+    permission_required = "paweljong.create_info_mailing_rule"
     template_name = "paweljong/info_mailing/create.html"
     success_url = reverse_lazy("info_mailings")
     success_message = _("The info mailing has been created.")
@@ -805,7 +816,7 @@ class InfoMailingEditView(PermissionRequiredMixin, AdvancedEditView):
 
     model = InfoMailing
     form_class = EditInfoMailingForm
-    permission_required = "paweljong.edit_info_mailing"
+    permission_required = "paweljong.change_info_mailing_rule"
     template_name = "paweljong/info_mailing/edit.html"
     success_url = reverse_lazy("info_mailings")
     success_message = _("The info mailing has been saved.")
@@ -815,7 +826,79 @@ class InfoMailingDeleteView(PermissionRequiredMixin, AdvancedDeleteView):
     """Delete view for info mailings."""
 
     model = InfoMailing
-    permission_required = "paweljong.delete_info_mailing"
+    permission_required = "paweljong.delete_info_mailing_rule"
     template_name = "core/pages/delete.html"
     success_url = reverse_lazy("info_mailings")
     success_message = _("The info mailing has been deleted.")
+
+
+class SendMailFromRegistration(PermissionRequiredMixin, FormView):
+
+    template_name = "paweljong/event_registration/notification.html"
+    permission_required = "paweljong.send_notification_mail_rule"
+    form_class = RegistrationNotificationForm
+    success_url = reverse_lazy("registrations")
+
+    def form_valid(self, form):
+
+        registration = EventRegistration.objects.get(id=self.kwargs["pk"])
+
+        context = {}
+        recipient_list = []
+        context["subject"] = form.cleaned_data["subject"]
+        context["registration"] = registration
+        context["body"] = form.cleaned_data["text"]
+
+        if form.cleaned_data["reply_to"]:
+            reply_to = form.cleaned_data["reply_to"]
+        else:
+            reply_to = form.cleaned_data["sender"]
+        if form.cleaned_data["send_to_person"]:
+            recipient_list.append(registration.person.email)
+        if form.cleaned_data["send_to_guardians"]:
+            recipient_list.append(registration.person.guardians.first().email)
+
+        send_templated_mail(
+            template_name="event_notification",
+            from_email=get_site_preferences()["mail__address"],
+            recipient_list=recipient_list,
+            headers={
+                "reply_to": reply_to,
+            },
+            context=context,
+        )
+
+        return super().form_valid(self)
+
+
+class RegistrationStateListView(PermissionRequiredMixin, SingleTableView):
+    """Table of all terms."""
+
+    model = RegistrationState
+    table_class = RegistrationStatesTable
+    permission_required = "paweljong.view_registration_states_rule"
+    template_name = "paweljong/registration_state/list.html"
+
+
+@method_decorator(never_cache, name="dispatch")
+class RegistrationStateCreateView(PermissionRequiredMixin, AdvancedCreateView):
+    """Create view for terms."""
+
+    model = RegistrationState
+    form_class = RegistrationStatesForm
+    permission_required = "paweljong.create_registration_states_rule"
+    template_name = "paweljong/registration_state/create.html"
+    success_url = reverse_lazy("registration_states")
+    success_message = _("The term has been created.")
+
+
+@method_decorator(never_cache, name="dispatch")
+class RegistrationStateEditView(PermissionRequiredMixin, AdvancedEditView):
+    """Edit view for terms."""
+
+    model = RegistrationState
+    form_class = RegistrationStatesForm
+    permission_required = "paweljong.change_registration_states_rule"
+    template_name = "paweljong/registration_state/edit.html"
+    success_url = reverse_lazy("registration_states")
+    success_message = _("The term has been saved.")
